from django.contrib import admin

from . import models


class FacebookAccountAdmin(admin.ModelAdmin):
    list_display = (
        'user',
        'fb_user_id',
        'date_created',
    )
    search_fields = ('user__username', 'user__email', 'fb_user_id')
    raw_id_fields = ('user',)


admin.site.register(models.FacebookAccount, FacebookAccountAdmin)



import pytest
import responses

from django.contrib.auth import authenticate
from django.contrib.auth.models import AnonymousUser
from django.contrib.sessions.middleware import SessionMiddleware
from django.test import RequestFactory
from mixer.backend.django import mixer

from .. import settings
from .. import utils

pytestmark = pytest.mark.django_db


def test_success_handler_default():
    req = RequestFactory().get('/')
    req.user = AnonymousUser()
    SessionMiddleware().process_request(req)
    req.session.save()

    # before our test, the request user is anonymous
    assert req.user.is_authenticated is False

    user = mixer.blend('auth.User')
    user.backend = 'facebook_login.auth_backends.FacebookAuthBackend'
    res = utils.success_handler_default(req, user)
    # after our test, the request has the authenticated user attached
    assert req.user == user
    assert req.user.is_authenticated is True


@responses.activate
def test_get_app_access_token():
    url = (f'{settings.API_BASE_URL}/oauth/access_token?'
           f'client_id={settings.APP_ID}&'
           f'client_secret={settings.APP_SECRET}&'
           f'grant_type=client_credentials')

    responses.add(
        responses.GET,
        url,
        json={'access_token': 'test-access-token'},
        status=200)

    res = utils.get_app_access_token()
    assert res == 'test-access-token'

    # TODO: test a bad response


@responses.activate
def test_debug_token():
    user_access_token = 'test-user-access-token'
    app_access_token = 'test-app-access-token'

    url = (f'{settings.API_BASE_URL}/debug_token?'
           f'input_token={user_access_token}&'
           f'access_token={app_access_token}')

    responses.add(
        responses.GET,
        url,
        json={'data': {
            'user_id': 'test-fb-user-id'
        }},
        status=200)

    res = utils.debug_token(user_access_token, app_access_token)
    assert res['data']['user_id'] == 'test-fb-user-id'

    # TODO: test a bad response



import pytest
import responses

from django.contrib.auth.models import User
from mixer.backend.django import mixer

from . import utils as test_utils
from .. import auth_backends
from .. import models

pytestmark = pytest.mark.django_db


class TestFacebookAuthBackend:

    def test_authenticate_none(self):
        backend = auth_backends.FacebookAuthBackend()
        req = test_utils.get_request()

        res = backend.authenticate(req, email=None, token=None)
        assert res is None, (
            'Should return `None`, if no credentials are given')

        res = backend.authenticate(req, email='test@example.com', token=None)
        assert res is None, ('Should return `None`, if no token is given')

        res = backend.authenticate(req, email=None, token='test-token')
        assert res is None, ('Should return `None`, if no token is given')

    @responses.activate
    def test_authenticate_bad_token(self):
        test_utils.setup_access_token_responses()
        test_utils.setup_debug_token_responses_error()
        backend = auth_backends.FacebookAuthBackend()
        req = test_utils.get_request()

        result = backend.authenticate(
            req, email='test@example.com', token='test-user-token')
        assert result is None, (
            'Should return `None` if an invalid token was given')

    @responses.activate
    def test_authenticate_known_facebook_user(self):
        test_utils.setup_access_token_responses()
        test_utils.setup_debug_token_responses_success()
        backend = auth_backends.FacebookAuthBackend()
        req = test_utils.get_request()

        user = mixer.blend('auth.User', email='test@example.com')
        fb_account = mixer.blend(
            'facebook_login.FacebookAccount',
            user=user,
            fb_user_id='test-fb-user-id')

        result = backend.authenticate(
            req, email='test@example.com', token='test-user-token')
        assert result == user, (
            'Should return the Django user that has the same email as the'
            ' given Facebook user')
        assert models.FacebookAccount.objects.all().count() == 1, (
            'Should create a FacebookAccount instance')
        assert models.FacebookAccount.objects.all()[0].user == user, (
            'Should tie FacebookAccount instance to Django user')

    @responses.activate
    def test_authenticate_connect_fb_user_with_known_django_user(self):
        test_utils.setup_access_token_responses()
        test_utils.setup_debug_token_responses_success()
        backend = auth_backends.FacebookAuthBackend()
        req = test_utils.get_request()

        user = mixer.blend('auth.User', email='test@example.com')
        result = backend.authenticate(
            req, email='test@example.com', token='test-user-token')
        assert result == user, ('Should return the Django user ')

    @responses.activate
    def test_authenticate_signup_completely_unknown_user(self):
        test_utils.setup_access_token_responses()
        test_utils.setup_debug_token_responses_success()
        backend = auth_backends.FacebookAuthBackend()
        req = test_utils.get_request()

        result = backend.authenticate(
            req, email='test@example.com', token='test-user-token')
        assert User.objects.all().count() == 1, (
            'Should create new Django user')
        assert models.FacebookAccount.objects.all().count() == 1, (
            'Should create new FacebookAccount')
        assert models.FacebookAccount.objects.all()[
            0].user == User.objects.all()[0], (
                'Should tie new instances together')

    def test_get_user(self):
        backend = auth_backends.FacebookAuthBackend()
        user = mixer.blend('auth.User')
        res = backend.get_user(666)
        assert res is None, (
            'Should return `None` when an unknown user-id was provided')

        res = backend.get_user(user.pk)
        assert res == user, ('Should return the user for the given PK')



from django.conf import settings

# Mandatory settings that have to be provided by the user:
APP_ID = settings.FB_LOGIN_APP_ID
APP_SECRET = settings.FB_LOGIN_APP_SECRET

# Optional settings that can be overridden by the user:
API_BASE_URL = getattr(settings, 'FB_LOGIN_API_BASE_URL',
                       'https://graph.facebook.com/v3.1')

SUCCESS_HANDLER = getattr(settings, 'FB_LOGIN_SUCCESS_HANDLER',
                          'facebook_login.utils.success_handler_default')



import uuid

from django.contrib.auth import get_user_model

from .models import FacebookAccount
from .utils import debug_token, get_app_access_token


class FacebookAuthBackend:

    def authenticate(self, request, email=None, token=None):
        """
        Tries to authenticate the given email and Facebook user access token.

        NOTE: Unlike most other custom authentication backends, this backend
        will create a new user if the given email/token are not found in the
        Django DB, yet. After validating the token, we can assume that the
        email/token combination is valid, because that is what Facebook
        returned to us.

        :request: A WSGIRequest instance.
        :email: The email of the user as returned by the Facebook login popup.
        :token: The user access token as returned by the Facebook login popup.
        :returns: A User instance or `None`.

        """
        if not email or not token:
            return None
        app_access_token = get_app_access_token()
        token_info = debug_token(token, app_access_token)
        if token_info.get('error'):
            return None

        # Step1: See if we already know this Facebook user...
        user_id = token_info['data']['user_id']
        user = None
        fb_account = None
        try:
            fb_account = FacebookAccount.objects.get(fb_user_id=user_id)
            if fb_account.user:
                return fb_account.user
        except FacebookAccount.DoesNotExist:
            pass

        # Step2: Unknown FB user, see if we already know this email...
        User = get_user_model()
        try:
            user = User.objects.get(email=email)
            FacebookAccount.objects.filter(user=user).delete()
            fb_account = FacebookAccount.objects.create(
                user=user, fb_user_id=user_id)
            return user
        except User.DoesNotExist:
            pass

        # Step3: Unknown FB user and email, let's create a new user...
        user = User.objects.create(username=str(uuid.uuid4()), email=email)
        fb_account = FacebookAccount.objects.create(
            user=user, fb_user_id=user_id)
        return user

    def get_user(self, user_id):
        User = get_user_model()
        try:
            return User.objects.get(pk=user_id)
        except User.DoesNotExist:
            return None



from builtins import str
from builtins import object
import graphene
import json
import requests

from django.contrib.auth import authenticate, get_user_model

from .models import FacebookAccount
from .utils import debug_token, get_app_access_token, success_handler


class FacebookAuthMutation(graphene.Mutation):

    class Arguments(object):
        email = graphene.String()
        access_token = graphene.String()

    status = graphene.Int()
    form_errors = graphene.String()
    extra = graphene.types.json.JSONString()

    @staticmethod
    def mutate(root, info, **args):
        user_access_token = args.get('access_token')
        email = args.get('email')

        if not email:
            return FacebookAuthMutation(
                status=400,
                form_errors=json.dumps({
                    'facebook': [
                        'Facebook login failed. You have not granted access to'
                        ' your email address.'
                    ]
                }),
                extra=None,
            )

        user = authenticate(
            request=info.context, email=email, token=user_access_token)
        if user is None:
            return FacebookAuthMutation(
                status=400,
                form_errors=json.dumps({
                    'facebook': ['Facebook login failed.']
                }),
                extra=None,
            )

        extra = success_handler(info.context, user)
        extra = json.dumps(extra) if extra else None

        return FacebookAuthMutation(status=200, form_errors=None, extra=extra)


class Mutation(object):
    facebook_auth = FacebookAuthMutation.Field()



import pytest
from mixer.backend.django import mixer

from .. import models

pytestmark = pytest.mark.django_db


class TestFacebookAccount(object):

    def test_model(self):
        obj = mixer.blend('facebook_login.FacebookAccount')
        assert obj.pk, ('Should save an instance')

    def test_str(self):
        user = mixer.blend('auth.User', email='test@example.com')
        obj = mixer.blend('facebook_login.FacebookAccount', user=user)
        result = str(obj)
        expected = str(user)
        assert result == expected, ('Should return the user email')



import requests

from django.contrib.auth import authenticate, login
from django.utils.module_loading import import_string

from . import settings


def success_handler_default(request, user):
    """
    Logs in the user that has been queried or created by the
    FacebookAuthMutation.

    """
    login(request, user)
    return None


def success_handler(request, user):  # pragma: nocover
    # too trivial to test
    """Imports and then calls the success handler function."""
    success_handler_func = import_string(settings.SUCCESS_HANDLER)
    return success_handler_func(request, user)


def get_app_access_token():
    """
    Gets an access token from Facebook.

    We need this token in order to make more Facebook API requests.

    """
    resp = requests.get(f'{settings.API_BASE_URL}/oauth/access_token?'
                        f'client_id={settings.APP_ID}&'
                        f'client_secret={settings.APP_SECRET}&'
                        f'grant_type=client_credentials').json()
    return resp['access_token']


def debug_token(user_access_token, app_access_token):
    """
    Validates a given user access token and returns user information.

    On the client, the Facebook login returns a userID and a token. We only
    need the userID to hook it up with our own user model but we need to verify
    that the token that the client has sent us is indeed valid, otherwise
    anyone could impersonate anyone by just sending us their Facebook userID.

    """
    resp = requests.get(f'{settings.API_BASE_URL}/debug_token?'
                        f'input_token={user_access_token}&'
                        f'access_token={app_access_token}')
    return resp.json()



import pytest
import responses

from . import utils
from .. import schema

pytestmark = pytest.mark.django_db


class TestFacebookAuthMutation:

    @responses.activate
    def test_mutate(self):
        m = schema.FacebookAuthMutation()
        utils.setup_access_token_responses()
        utils.setup_debug_token_responses_success()
        info = utils.InfoFactory()
        res = m.mutate(None, info)
        assert res.status == 400, ('Should return 400 if no email was given')

        res = m.mutate(None, info, email='test@example.com')
        assert res.status == 400, (
            'Should return 400 if the Facebook login failed (i.e. becuase no'
            ' token or an invalid token was given)')

        res = m.mutate(
            None, info, email='test@example.com', access_token='test-token')

        assert res.status == 200, (
            'Should return 200 if the Facebook login succeeded')

        assert res.extra is None, (
            'Should return None if success handler not specified')

    @responses.activate
    def test_custom_success_handler_with_extra_data(self):
        # monkeypatching the setting
        from facebook_login import settings
        old = settings.SUCCESS_HANDLER
        settings.SUCCESS_HANDLER = 'facebook_login.tests.utils.custom_success_handler'

        m = schema.FacebookAuthMutation()
        utils.setup_access_token_responses()
        utils.setup_debug_token_responses_success()
        info = utils.InfoFactory()

        res = m.mutate(
            None, info, email='test@example.com', access_token='test-token')

        assert res.status == 200, (
            'Should return 200 if the Facebook login succeeded')

        assert 'token' in res.extra, (
            'Should return extra data from success handler')

        # undo monkeypatching
        settings.SUCCESS_HANDLER = old



import os
import facebook_login

from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

install_requires = open('requirements.txt').read().splitlines()

dev_requires = [
    'pytest',
    'pytest-django',
    'pytest-cov',
    'responses',
    'fabric',
    'mixer',
]

setup(
    name='django-facebook-login',
    version=facebook_login.__version__,
    packages=find_packages(),
    include_package_data=True,
    license='MIT License',
    description=
    'A GraphQL endpoint and authentication backend to signup or login a valid user access token from Facebook',
    long_description=README,
    long_description_content_type='text/markdown',
    url='https://github.com/theartling/django-facebook-login/',
    author='Martin Brochhaus',
    author_email='mbrochh@gmail.com',
    install_requires=install_requires,
    extras_require={
        'dev': dev_requires,
    },
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.0',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



from django.conf import settings
from django.db import models


class FacebookAccount(models.Model):
    """

    :user: FK to the Django user that this Facebook account is tied to.
    :fb_user_id: The Facebook user-id.
    :date_created: Date when this instance was created.

    """
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL, on_delete=models.CASCADE)
    fb_user_id = models.CharField(max_length=1024)
    date_created = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return f'{self.user}'



import responses

from django.contrib.auth import login
from django.contrib.auth.models import AnonymousUser
from django.contrib.sessions.middleware import SessionMiddleware
from django.test import RequestFactory

from .. import settings


class InfoFactory(object):
    context = None

    def __init__(self, user=None, *args, **kwargs):
        if user is None:
            user = AnonymousUser()
        self.context = RequestFactory().get('/')
        self.context.user = user
        SessionMiddleware().process_request(self.context)
        self.context.session.save()


def get_request():
    req = RequestFactory().get('/')
    SessionMiddleware().process_request(req)
    req.session.save()
    return req


def setup_access_token_responses():
    """
    We assume that we have hooked up the app correctly and have correct
    APP_ID and APP_SECRET in our local_settings.

    """
    url = (f'{settings.API_BASE_URL}/oauth/access_token?'
           f'client_id={settings.APP_ID}&'
           f'client_secret={settings.APP_SECRET}&'
           f'grant_type=client_credentials')
    responses.add(
        responses.GET,
        url,
        json={'access_token': 'test-access-token'},
        status=200)


def setup_debug_token_responses_success():
    """
    We assume that the user gave a valid Facebook user access token.

    Facebook would then return some data about that token. The part that
    interests us is the facebook `user_id`.

    """
    url = (f'{settings.API_BASE_URL}/debug_token?'
           f'input_token=test-user-token&'
           f'access_token=test-access-token')
    responses.add(
        responses.GET,
        url,
        json={'data': {
            'user_id': 'test-fb-user-id'
        }},
        status=200)


def setup_debug_token_responses_error():
    """
    We assume that the user provided an invalid Facebook user access token.

    In this case, Facebook will return something that contains the key
    `error`. If we find that key, we don't want to login the user!

    """
    url = (f'{settings.API_BASE_URL}/debug_token?'
           f'input_token=test-user-token&'
           f'access_token=test-access-token')
    responses.add(
        responses.GET,
        url,
        json={
            'data': {
                'user_id': 'something'
            },
            'error': 'Oh Oh!'
        },
        status=200)


def custom_success_handler(request, user):
    login(request, user)
    return {'extra': 'token'}
